"""
Some functions to help build motions and postures.
"""

#TODO: everything should have an optional write=False 

def kfm2mot(kfmname, motname):
    """Convert a KFM file (Sony MEdit) to MOT (Tekkotsu)."""
    names = ["NECK:tilt", "NECK:pan~", "NECK:nod~", "MOUTH~~~~" , "EAR:left~", "EAR:right", "LFr:rotor", "LFr:elvtr", "LFr:knee~", "LBk:rotor", "LBk:elvtr", "LBk:knee~", "RFr:rotor", "RFr:elvtr", "RFr:knee~", "RBk:rotor", "RBk:elvtr", "RBk:knee~", "TAIL:tilt", "TAIL:pan~"]
    kfmdata = [line.replace("\n", "") for line in file(kfmname)]
    kfmdata = zip(kfmdata[::3], [s.split() for s in kfmdata[2::3]])
    motfile = file(motname, "w")

    motfile.write("#MSq\nradians\n")
    for (delay, jointvalues) in kfmdata:
        motfile.write("advanceTime\t%s\n" % (int(delay)*16))
        for (name, value) in zip(names, jointvalues):
            motfile.write("%s\t%.4f\n" % (name, float(value)))
    motfile.write("#END")

def pos2mot(posfiles, motname, advtime=150):
    """Convert a list of posture files to a motion file (default advtime=150)."""
    motfile = file(motname, "w")
    motfile.write("#MSq\nradians\n")
    for pos in posfiles:
        lines = file(pos).readlines()
        lines = lines[1:-1]
        lines = [map(lambda s: s.strip(), line.split("\t")) for line in lines]
        motfile.write("advanceTime\t%s\n" % advtime)
        for (name, value, weight) in lines:
            motfile.write("%s\t%.4f\n" % (name, float(value)))
    motfile.write("#END")
    motfile.close()

def mot2pos(motname):
    """Splits a motion into separate posture files"""
    lines = file(motname, "w")
    raise NotImplementedError()

def join_mots(motfiles, motname):
    """Joins a list of motion files together into one motion."""
    motfile = file(motname, "w")
    motfile.write("#MSq\nradians\n")
    for f in motfiles:
        motfile.writelines(file(f).readlines()[2:-1])
    motfile.write("#END")
    motfile.close()

def set_constant(filename, joint, value):
    """Sets a joint constant throughout each posture in a motion."""
    lines = file(filename).readlines()
    for (i,line) in enumerate(lines):
        if line.startswith(joint):
            lines[i] = joint + "\t" + value + "\n"
    file(filename, "w").writelines(lines)

## Remove joint commands ######################################################

def remove_joints(jointnames, filename):
    """Returns the text of the motion/posture without setting any joints in 'jointnames'.
    For example:
        >>> remove_joints(["EAR", "TAIL", "MOUTH"], "e:/data/motion/test.mot")
    would return the text of test.mot without any commands setting the ears, tail, or mouth
    Note that the joints in jointnames can be complete joint names or prefixes (e.g. "L" will match "LBk..." and "LFr...")"""
    #TODO: make an optional named parameter to rewrite the file (e.g. strip_joints(jointnames, filename, write=False))
    def remove_joint(jointname, lines):
        return (line for line in lines if not line.startswith(jointname))

    lines = file(filename).readlines()
    if isinstance(jointnames, list):
        for name in jointnames:
            lines = remove_joint(name, lines)
    elif isinstance(jointnames, str):
        lines = remove_joint(jointnames, lines)
    return "".join(lines)

def remove_leds(filename):
    """Strips out lines that control the LEDs."""
    return remove_joints("LED", filename)

def remove_legs(filename):
    """Strips out lines that control the legs."""
    return remove_joints(["LB","RB","LF","RF"], filename)

def remove_head(filename):
    """Strips out lines that control the head."""
    return remove_joints("NECK", filename)

def remove_tail(filename):
    """Strips out lines that control the tail."""
    return remove_joints("TAIL", filename)

def remove_ears(filename):
    """Strips out lines that control the ears."""
    return remove_joints("EAR", filename)

def remove_mouth(filename):
    """Strips out lines that control the mouth."""
    return remove_joints("MOUTH", filename)